/*eslint no-var: "error"*/
/*eslint no-unused-vars: "off" */
/*eslint-env es6*/
// Put this file in path/to/plugin/amd/src
// You can call it anything you like

define(['jquery', 'core/str', 'core/ajax', 'core/config', 'block_gradelevel/renderbadge', 'block_gradelevel/handlers',
    'block_gradelevel/debugger', 'core/modal_factory', 'core/modal_events'],
    function ($, str, ajax, mdlcfg, renderbadge, handlers, Debugger, ModalFactory, ModalEvents)
    {

    let debug = Debugger("skilleditor");
    debug.enable();

    let $skillset_div = null;
    const uploadFilter = ".png,.svg";

    let self = {
        init: function init() {
            // Find skill table and skill id
            $skillset_div = $("div#skill_set");
            // Run initialization code on all items.
            debug.info("Initializing editors");
            $skillset_div.find("li.skill_info").each(function () {
                self.initialize_item(this);
            });

            debug.info("Attaching function to addskill link");
            $skillset_div.find("a[data-action='addskill']").on('click', function (e) {
                let promises = ajax.call([{
                    methodname: 'block_gradelevel_add_skill',
                    args: { name: null, icon: null },
                }]);

                // and link promise returns to callbacks
                promises[0].fail(handlers.fail_report_exception).done(function (data) {
                    let $newskill = $(data.html);
                    $skillset_div.find('ul').append($newskill);
                    $newskill.each(function () {
                        self.initialize_item(this);
                    });
                    $newskill.find('figure.levelbadge').each(function () {
                        renderbadge.setup_badge(this);
                    });

                });
            });

        },
        initialize_item: function initialize_item(li) {
            let $li = $(li);
            let skill_id = $li.data('id');

            // Attach edit name pencil
            $li.find("a[data-action='editname']").on('click', function (e) {
                let $span = $li.find("figcaption span[data-type='label']");
                if ($span.length > 0) {
                    let name = $span.text();
                    let $input = $("<input type='text'/>").val(name);// value='" + name + "' />");

                    $span.replaceWith($input);
                    $input.on("keyup", function input_keypress(e) {
                        let keycode = (e.keyCode ? e.keyCode : e.which);
                        if (keycode == '13') {
                            debug.info("Saving name");
                            // save the new name
                            let new_name = $input.val();

                            let promises = ajax.call([{
                                methodname: 'block_gradelevel_update_skill',
                                args: { id: skill_id, name: new_name, icon: null },
                            }]);

                            // and link promise returns to callbacks
                            promises[0].fail(handlers.fail_report_exception).done(function (data) {
                                $input.replaceWith($("<span data-type='label'>" + data.name + "</span>"));
                            });

                        }
                        if (keycode == '27') {
                            debug.info("Cancelled editing");
                            $input.replaceWith($("<span data-type='label'>" + name + "</span>"));
                        }
                    }).on("focusout", function input_cancel(e) {
                        debug.info("Cancelled editing");
                        $input.replaceWith($("<span data-type='label'>" + name + "</span>"));
                    });
                    $input.focus();

                }
                else {
                    let $textinput = $li.find("figurecaption input[type=text]");

                }
            });

            // Attach edit icon pencil
            $li.find("a[data-action='editicon']").on('click', function (e) {
                let $uploader = $("<input type='file' style='display: none;' accept='" + uploadFilter + "'/>");

                $uploader.on('change', function (e) {
                    e.stopPropagation();
                    e.preventDefault();
                    let file = e.originalEvent.target.files[0];
                    let fr = new FileReader();
                    $(fr).on('load', function (e) {
                        let dataURI = e.target.result;

                        let promises = ajax.call([{
                            methodname: 'block_gradelevel_update_skill',
                            args: { id: skill_id, name: null, icon: dataURI },
                        }]);

                        // and link promise returns to callbacks
                        promises[0].fail(handlers.fail_report_exception).done(function (data) {

                            let $figure = $li.find("figure.levelbadge");

                            $figure.empty();
                            $figure.append($("<img style='display:none;' src='"+data.icon+"'/>"));

                            $figure.each(function () {
                                renderbadge.setup_badge(this);
                            });
                        });


                        $uploader.remove(); // delete the uploader after it's done
                    });
                    fr.readAsDataURL(file);

                });
                $li.append($uploader);
                $uploader.trigger('click');

            });

            // Attach edit levels link
            $li.find("a[data-action='editlevels']").attr('href',
                mdlcfg.wwwroot + "/blocks/gradelevel/cfg_skilllevels.php?skill_id=" + skill_id);

            // Attach delete function to any existing functional delete link
            $li.find("a[data-action='deleteskill']").each(function () {
                let $trigger = $(this);
                // Attach delete button code
                ModalFactory.create({
                    type: ModalFactory.types.SAVE_CANCEL,
                    title: str.get_string('title_confirmdelete', 'block_gradelevel'),
                    body: str.get_string('dialog_confirmdeleteskill', 'block_gradelevel'),
                }, $trigger).done(function (modal) {
                    // Do what you want with your new modal.
                    debug.info("Modal done", modal);
                    modal.setSaveButtonText(str.get_string('delete', 'core'));
                    modal.getRoot().on(ModalEvents.save, function (e) {
                        debug.info("Delete Confirmed");
                        //e.preventDefault(); if you don't want to close the window

                        // call the delete function
                        let promises = ajax.call([{
                            methodname: 'block_gradelevel_delete_skill',
                            args: { id: skill_id},
                        }]);

                        // and link promise returns to callbacks
                        promises[0].fail(handlers.fail_report_exception).done(function (data) {

                            if (data.deleted) {
                                $li.remove();
                            }

                        });
                    });
                });
            });

        },
    };
    return self;
});